'use strict';

var List = require("bs-platform/lib/js/list.js");
var $$Array = require("bs-platform/lib/js/array.js");
var Block = require("bs-platform/lib/js/block.js");
var Format = require("bs-platform/lib/js/format.js");
var $$String = require("bs-platform/lib/js/string.js");
var Pervasives = require("bs-platform/lib/js/pervasives.js");
var Caml_format = require("bs-platform/lib/js/caml_format.js");
var Caml_option = require("bs-platform/lib/js/caml_option.js");
var Caml_exceptions = require("bs-platform/lib/js/caml_exceptions.js");
var Caml_js_exceptions = require("bs-platform/lib/js/caml_js_exceptions.js");
var Caml_builtin_exceptions = require("bs-platform/lib/js/caml_builtin_exceptions.js");

var WrongPlatform = Caml_exceptions.create("AST.WrongPlatform");

function unavailable_on(target, name) {
  throw [
        WrongPlatform,
        target,
        name
      ];
}

function tOfJs(param) {
  throw [
        WrongPlatform,
        /* Native */-693027401,
        "tOfJs"
      ];
}

function expressionOfJs(param) {
  throw [
        WrongPlatform,
        /* Native */-693027401,
        "expressionOfJs"
      ];
}

function to_yojson(param) {
  throw [
        WrongPlatform,
        /* JavaScript */500616045,
        "to_yojson"
      ];
}

function of_yojson(param) {
  throw [
        WrongPlatform,
        /* JavaScript */500616045,
        "of_yojson"
      ];
}

function expression_to_yojson(param) {
  throw [
        WrongPlatform,
        /* JavaScript */500616045,
        "expression_to_yojson"
      ];
}

function expression_of_yojson(param) {
  throw [
        WrongPlatform,
        /* JavaScript */500616045,
        "expression_of_yojson"
      ];
}

function flagToJs(param) {
  return {
          name: param[/* name */0],
          payload: param[/* payload */1]
        };
}

function flagFromJs(param) {
  return /* record */[
          /* name */param.name,
          /* payload */param.payload
        ];
}

function expressionToJs(param) {
  return {
          count: param[/* count */0],
          cmd: param[/* cmd */1],
          rev_args: param[/* rev_args */2]
        };
}

function expressionFromJs(param) {
  return /* record */[
          /* count */param.count,
          /* cmd */param.cmd,
          /* rev_args */param.rev_args
        ];
}

function tToJs(param) {
  return {
          expressions: param[/* expressions */0]
        };
}

function tFromJs(param) {
  return /* record */[/* expressions */param.expressions];
}

function make_expression(count, cmd, rev_args) {
  return /* record */[
          /* count */count !== undefined ? Caml_format.caml_int_of_string(count) : 1,
          /* cmd */cmd,
          /* rev_args */rev_args
        ];
}

function pipe_last(from, into) {
  var match = into[/* rev_args */2];
  var new_args;
  if (match) {
    var match$1 = match[0];
    if (match$1.tag) {
      var flg = match$1[0];
      var match$2 = flg[/* payload */1];
      if (typeof match$2 === "number") {
        if (match$2 !== 0) {
          var new_flag = /* record */[
            /* name */flg[/* name */0],
            /* payload : Unresolved */0
          ];
          new_args = /* :: */[
            /* Flag */Block.__(1, [new_flag]),
            match[1]
          ];
        } else {
          new_args = Pervasives.failwith("unreachable: pipe_last, unresolved-flag in last position");
        }
      } else {
        new_args = into[/* rev_args */2];
      }
    } else {
      new_args = into[/* rev_args */2];
    }
  } else {
    new_args = /* [] */0;
  }
  var new_args_000 = /* Positional */Block.__(0, [/* Sub */Block.__(0, [from])]);
  var new_args$1 = /* :: */[
    new_args_000,
    new_args
  ];
  return /* record */[
          /* count */into[/* count */0],
          /* cmd */into[/* cmd */1],
          /* rev_args */new_args$1
        ];
}

function copy_string_or_subexpr(sos) {
  if (sos.tag) {
    return /* Literal */Block.__(1, [$$String.copy(sos[0])]);
  } else {
    return /* Sub */Block.__(0, [copy_expression(sos[0])]);
  }
}

function copy_flag(flg) {
  var match = flg[/* payload */1];
  return /* record */[
          /* name */flg[/* name */0],
          /* payload */typeof match === "number" ? (
              match !== 0 ? /* Absent */1 : /* Unresolved */0
            ) : /* Resolved */[copy_string_or_subexpr(match[0])]
        ];
}

function copy_arg(arg) {
  if (arg.tag) {
    return /* Flag */Block.__(1, [copy_flag(arg[0])]);
  } else {
    return /* Positional */Block.__(0, [copy_string_or_subexpr(arg[0])]);
  }
}

function copy_expression(expr) {
  return /* record */[
          /* count */expr[/* count */0],
          /* cmd */copy_string_or_subexpr(expr[/* cmd */1]),
          /* rev_args */List.map(copy_arg, expr[/* rev_args */2])
        ];
}

function copy(ast) {
  return /* record */[/* expressions */$$Array.map(copy_expression, ast[/* expressions */0])];
}

function pp_bs(ast) {
  var obj = tToJs(ast);
  console.log(Caml_option.undefined_to_opt(JSON.stringify(obj)));
  return /* () */0;
}

function pp_native(ast) {
  throw [
        WrongPlatform,
        /* JavaScript */500616045,
        "to_yojson"
      ];
}

function pp(ast) {
  try {
    return pp_bs(ast);
  }
  catch (raw_exn){
    var exn = Caml_js_exceptions.internalToOCamlException(raw_exn);
    if (exn[0] === WrongPlatform) {
      if (exn[1] !== -693027401) {
        throw exn;
      }
      return pp_native(ast);
    } else {
      throw exn;
    }
  }
}

function pp_expression_bs(expr) {
  var obj = expressionToJs(expr);
  console.log(Caml_option.undefined_to_opt(JSON.stringify(obj)));
  return /* () */0;
}

function pp_expression_native(expr) {
  throw [
        WrongPlatform,
        /* JavaScript */500616045,
        "expression_to_yojson"
      ];
}

function pp_expression(ast) {
  try {
    return pp_expression_bs(ast);
  }
  catch (raw_exn){
    var exn = Caml_js_exceptions.internalToOCamlException(raw_exn);
    if (exn[0] === WrongPlatform) {
      if (exn[1] !== -693027401) {
        throw exn;
      }
      return pp_expression_native(ast);
    } else {
      throw exn;
    }
  }
}

function is_literal(param) {
  if (param.tag) {
    return true;
  } else {
    return false;
  }
}

function get_literal_exn(param) {
  if (param.tag) {
    return param[0];
  } else {
    throw Caml_builtin_exceptions.not_found;
  }
}

function get_sub_exn(param) {
  if (param.tag) {
    throw Caml_builtin_exceptions.not_found;
  } else {
    return param[0];
  }
}

exports.WrongPlatform = WrongPlatform;
exports.unavailable_on = unavailable_on;
exports.tOfJs = tOfJs;
exports.expressionOfJs = expressionOfJs;
exports.to_yojson = to_yojson;
exports.of_yojson = of_yojson;
exports.expression_to_yojson = expression_to_yojson;
exports.expression_of_yojson = expression_of_yojson;
exports.flagToJs = flagToJs;
exports.flagFromJs = flagFromJs;
exports.expressionToJs = expressionToJs;
exports.expressionFromJs = expressionFromJs;
exports.tToJs = tToJs;
exports.tFromJs = tFromJs;
exports.make_expression = make_expression;
exports.pipe_last = pipe_last;
exports.copy_string_or_subexpr = copy_string_or_subexpr;
exports.copy_flag = copy_flag;
exports.copy_arg = copy_arg;
exports.copy_expression = copy_expression;
exports.copy = copy;
exports.pp_bs = pp_bs;
exports.pp_native = pp_native;
exports.pp = pp;
exports.pp_expression_bs = pp_expression_bs;
exports.pp_expression_native = pp_expression_native;
exports.pp_expression = pp_expression;
exports.is_literal = is_literal;
exports.get_literal_exn = get_literal_exn;
exports.get_sub_exn = get_sub_exn;
/* Format Not a pure module */
