'use strict';

var Curry = require("bs-platform/lib/js/curry.js");
var Lexer = require("./lexer.bs.js");
var Lexing = require("bs-platform/lib/js/lexing.js");
var MenhirLib = require("./menhirLib.bs.js");
var Expression = require("./expression.bs.js");
var Caml_option = require("bs-platform/lib/js/caml_option.js");
var Caml_exceptions = require("bs-platform/lib/js/caml_exceptions.js");
var ParserAutomaton = require("./parserAutomaton.bs.js");
var Caml_js_exceptions = require("bs-platform/lib/js/caml_js_exceptions.js");
var Caml_builtin_exceptions = require("bs-platform/lib/js/caml_builtin_exceptions.js");

var ParseError = Caml_exceptions.create("Parser.ParseError");

function error_loctoken_exn(param) {
  if (param[0] === ParseError) {
    return param[1];
  } else {
    throw [
          Caml_builtin_exceptions.invalid_argument,
          "error_loctoken_exn: expects a ParseError"
        ];
  }
}

function parse($staropt$star, buf, p) {
  var exn = $staropt$star !== undefined ? $staropt$star : true;
  var parser = Curry._1(MenhirLib.Convert.Simplified.traditional2revised, p);
  var last_token = /* record */[/* contents : tuple */[
      /* EOF */7,
      Lexing.dummy_pos,
      Lexing.dummy_pos
    ]];
  var next_token = function (param) {
    var tok = Lexer.next_loc(buf);
    last_token[0] = tok;
    return tok;
  };
  try {
    return Caml_option.some(Curry._1(parser, next_token));
  }
  catch (raw_exn){
    var exn$1 = Caml_js_exceptions.internalToOCamlException(raw_exn);
    if (exn$1[0] === Lexer.LexError) {
      if (exn) {
        throw [
              Lexer.LexError,
              exn$1[1],
              exn$1[2]
            ];
      } else {
        return ;
      }
    } else if (exn$1 === ParserAutomaton.$$Error) {
      if (exn) {
        throw [
              ParseError,
              last_token[0]
            ];
      } else {
        return ;
      }
    } else {
      throw exn$1;
    }
  }
}

function parse_string(exn, s, p) {
  return parse(exn, Lexer.buffer_of_string(s), p);
}

function script(exn, buf) {
  return parse(exn, buf, ParserAutomaton.script);
}

function script_of_string(exn, str) {
  return parse_string(exn, str, ParserAutomaton.script);
}

function expression(exn, buf) {
  var match = parse(exn, buf, ParserAutomaton.expression);
  if (match !== undefined) {
    return Caml_option.some(Expression.dehydrate(match));
  }
  
}

function expression_of_string(exn, str) {
  var match = parse_string(exn, str, ParserAutomaton.expression);
  if (match !== undefined) {
    return Caml_option.some(Expression.dehydrate(match));
  }
  
}

var script_automaton = ParserAutomaton.script;

var expression_automaton = ParserAutomaton.expression;

exports.ParseError = ParseError;
exports.error_loctoken_exn = error_loctoken_exn;
exports.script = script;
exports.script_of_string = script_of_string;
exports.expression = expression;
exports.expression_of_string = expression_of_string;
exports.script_automaton = script_automaton;
exports.expression_automaton = expression_automaton;
exports.parse = parse;
exports.parse_string = parse_string;
/* Lexer Not a pure module */
