'use strict';

var AST = require("./aST.bs.js");
var List = require("bs-platform/lib/js/list.js");
var $$Array = require("bs-platform/lib/js/array.js");
var Block = require("bs-platform/lib/js/block.js");
var Curry = require("bs-platform/lib/js/curry.js");
var $$String = require("bs-platform/lib/js/string.js");
var MenhirLib = require("./menhirLib.bs.js");
var Pervasives = require("bs-platform/lib/js/pervasives.js");
var Caml_string = require("bs-platform/lib/js/caml_string.js");
var Caml_exceptions = require("bs-platform/lib/js/caml_exceptions.js");
var Caml_builtin_exceptions = require("bs-platform/lib/js/caml_builtin_exceptions.js");

var $$Error = Caml_exceptions.create("ParserAutomaton.MenhirBasics.Error");

function token2terminal(_tok) {
  if (typeof _tok === "number") {
    switch (_tok) {
      case /* SEMICOLON */0 :
          return 3;
      case /* PIPE */1 :
          return 8;
      case /* PAREN_OPEN */2 :
          return 9;
      case /* PAREN_CLOSE */3 :
          return 10;
      case /* FLAG_LONG_START */4 :
          return 12;
      case /* FLAGS_SHORT_START */5 :
          return 13;
      case /* EQUALS */6 :
          return 21;
      case /* EOF */7 :
          return 22;
      case /* COMMENT_OPEN */8 :
          return 24;
      case /* COMMENT_CLOSE */9 :
          return 25;
      case /* COLON */10 :
          return 27;
      case /* BARE_DOUBLE_DASH */11 :
          return 28;
      
    }
  } else {
    switch (_tok.tag | 0) {
      case /* URL_START */0 :
          return 1;
      case /* URL_REST */1 :
          return 2;
      case /* QUOTE_OPEN */2 :
          return 4;
      case /* QUOTE_ESCAPE */3 :
          return 5;
      case /* QUOTE_CLOSE */4 :
          return 6;
      case /* QUOTE_CHUNK */5 :
          return 7;
      case /* IDENTIFIER */6 :
          return 11;
      case /* ERR_UNEXPECTED_WHITESPACE */7 :
          return 14;
      case /* ERR_UNEXPECTED_QUOTE_ESCAPE */8 :
          return 15;
      case /* ERR_UNEXPECTED_QUOTE_CLOSE */9 :
          return 16;
      case /* ERR_UNEXPECTED_COMMENT_CLOSE */10 :
          return 17;
      case /* ERR_UNEXPECTED_CHARACTER */11 :
          return 18;
      case /* ERR_MISSING_DELIM_CLOSE */12 :
          return 19;
      case /* ERR_MISSING_COMMENT_CLOSE */13 :
          return 20;
      case /* COUNT */14 :
          return 23;
      case /* COMMENT */15 :
          return 26;
      
    }
  }
}

function token2value(_tok) {
  if (typeof _tok === "number") {
    return /* () */0;
  } else {
    return _tok[0];
  }
}

var default_reduction = /* tuple */[
  8,
  "\0\0\x0f\f\0\x18\0-\0\0\0\r6\b\n\x000!\x1f /\x00321.\t\0\0\x13\x12\0\x03\x04\0\x05\x06\x14\x16%\0'&5\0\0\x15\x11\0)\0*7($\0\x07\0\x0b\x1a\x01\0\x02\x1c\x004\x1b\0\x1d\0\x1e"
];

var error = /* tuple */[
  29,
  "\bP\x01\x10B\x80\b\x80\0\0\0\0\0\0\0\x85\0\x10\0\0\0\0!@\0\0\0\0\0\bP\x01\x10\x05\0\0\x02\x14\0D\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\xe0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x07\x80\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xb1\xf8\x04\x12\0\0\0\0\0\0\0\0\0\0\x01\x02\0\0\0\0\0\0\0\0\0\x02\x04\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x10\xac\x02\0\0\0\0\0\0\0\0\0\0\0\x05\x8f\xc0`\xa4(\0\x04\0\0\0\0\0\0\0X\xfc\x02\b\0\0\0\x16?\0\x82\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x02\x10\0@\0\0\0\0\0\0\x10\0\0\0\0\0\0\0\0\0\0\0\f(\x01\x88\0\0\0\0\0\0\0\0\0\x02\0\0\0\0\0\0\0\0!\0\x04\0\0\0\0\x04(\x01\x88\0\0\0\0"
];

var action = /* tuple */[
  /* tuple */[
    8,
    "\x96\x96\0\0\xce\0j\0\x96\t\x96\0\0\0\0\x12\0\0\0\0\0>\0\0\0\0\0*\x03\0\0\x1e\0\0J\0\0\0\0\0\xba\0\0\0\x03*\0\0V\0\x82\0\0\0\0(\0#\0\0\0\x9a\0\0\x0b\0\0(\0\x96\0"
  ],
  /* tuple */[
    16,
    "\0=\0v\0=\0=\0*\0\xeb\x002\0=\0=\0=\0=\0=\0=\0B\0F\0J\0N\x01\x07\0\0\0\x1e\0\xb6\0=\0r\0\xe2\0\x85\0\x1e\0\x82\0=\0*\0\x85\0\"\0\x85\0z\0~\0\x8a\0B\0F\0Z\0N\0\0\0\0\0\x1e\0a\0\x85\0r\0\0\0\xad\0\x1e\0\x8e\0\x96\0\0\0\xad\0\"\0\xad\0z\0\xad\0\xad\0\x1e\0\0\0\0\0\0\0\0\0\"\0\0\x006\0\xad\0r\0\0\0\xa9\0\x1e\0\0\0\x96\0\0\0\xa9\0\"\0\xa9\0z\0\xa9\0\xa9\x005\0\xe2\x005\0\0\0\0\x005\0\0\x005\0\xa9\x005\0\0\0\0\0\0\0\0\0\x96\0\0\0\0\0\x89\0e\x005\0a\x005\0\x89\0\x06\0\x89\0\x06\0~\0\x8a\0Y\0\0\0\0\0\0\0\0\0Y\0\0\0Y\0\x89\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x16"
  ]
];

var lhs = /* tuple */[
  8,
  "\x01\0\x1b\x1b\x1a\x1a\x19\x18\x18\x18\x17\x16\x16\x15\x15\x14\x14\x13\x13\x13\x13\x13\x12\x12\x11\x11\x10\x10\x0f\x0f\x0e\r\r\f\f\x0b\x0b\x0b\x0b\n\n\n\t\t\b\b\b\x07\x07\x07\x06\x05\x04\x03\x02"
];

var $$goto = /* tuple */[
  /* tuple */[
    8,
    " \x0e\0\0\x18\x004\0\0\0\x18\0\0\0\0\x07\0\0\0\0\0n\0\0\0\0\0\x07\0\0\0\n\0\0J\0\0\0\0\0 \0\0\0\0L\0\0R\0Z\0\0\0\0R\0\0\0\0\0\x14\0\0\0\0\0\\\0h\0"
  ],
  /* tuple */[
    8,
    "'(\x04\x15\x16\x10)135\x1a/\x04\x10\f?67\x04\"*\x05\n,-@AC\x03\x0f\x07\x05D\x05\x10<+\x058=\x1b',-'\x10\x10\0'\x10\x1c%/\x10\x04/\x000:/2\x18\x16E4\0<G\x19\0\0F\0\x05D"
  ]
];

var semantic_action = /* array */[
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_menhir_stack[/* semv */1],
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_menhir_stack[/* semv */1],
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_menhir_stack[/* semv */1],
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_menhir_stack[/* semv */1],
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv : () */0,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _v = /* Literal */Block.__(1, [_menhir_stack[/* semv */1]]);
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _v = /* Literal */Block.__(1, [_menhir_stack[/* semv */1]]);
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _v = /* Sub */Block.__(0, [_menhir_stack[/* semv */1]]);
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4][/* next */4];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */match[/* semv */1],
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_menhir_stack[/* semv */1],
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4][/* next */4];
      var _v = AST.pipe_last(match[/* semv */1], _menhir_stack[/* semv */1]);
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _menhir_s = _menhir_env[/* current */3];
      var _startpos = _menhir_stack[/* endp */3];
      return /* record */[
              /* state */_menhir_s,
              /* semv : [] */0,
              /* startp */_startpos,
              /* endp */_startpos,
              /* next */_menhir_stack
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var _v_000 = match[/* semv */1];
      var _v_001 = _menhir_stack[/* semv */1];
      var _v = /* :: */[
        _v_000,
        _v_001
      ];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _v = /* Flag */Block.__(1, [/* record */[
            /* name */_menhir_stack[/* semv */1],
            /* payload : Absent */1
          ]]);
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4][/* next */4];
      var _v = /* Flag */Block.__(1, [/* record */[
            /* name */match[/* semv */1],
            /* payload : Resolved */[_menhir_stack[/* semv */1]]
          ]]);
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _v = /* Literal */Block.__(1, [_menhir_stack[/* semv */1]]);
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var _v = /* Literal */Block.__(1, [match[/* semv */1] + _menhir_stack[/* semv */1]]);
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv : Literal */Block.__(1, ["--"]),
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _v = /* Literal */Block.__(1, [_menhir_stack[/* semv */1]]);
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _v = /* Sub */Block.__(0, [_menhir_stack[/* semv */1]]);
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _menhir_s = _menhir_env[/* current */3];
      var _startpos = _menhir_stack[/* endp */3];
      return /* record */[
              /* state */_menhir_s,
              /* semv */undefined,
              /* startp */_startpos,
              /* endp */_startpos,
              /* next */_menhir_stack
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _v = _menhir_stack[/* semv */1];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _menhir_s = _menhir_env[/* current */3];
      var _startpos = _menhir_stack[/* endp */3];
      return /* record */[
              /* state */_menhir_s,
              /* semv */undefined,
              /* startp */_startpos,
              /* endp */_startpos,
              /* next */_menhir_stack
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _v = _menhir_stack[/* semv */1];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv : [] */0,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_menhir_stack[/* semv */1],
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var _v_000 = match[/* semv */1];
      var _v = /* :: */[
        _v_000,
        /* [] */0
      ];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4][/* next */4];
      var _v_000 = match[/* semv */1];
      var _v_001 = _menhir_stack[/* semv */1];
      var _v = /* :: */[
        _v_000,
        _v_001
      ];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var _v = $$String.concat("", List.rev(match[/* semv */1]));
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_menhir_stack[/* semv */1],
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_menhir_stack[/* semv */1],
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _menhir_s = _menhir_env[/* current */3];
      var _startpos = _menhir_stack[/* endp */3];
      return /* record */[
              /* state */_menhir_s,
              /* semv : [] */0,
              /* startp */_startpos,
              /* endp */_startpos,
              /* next */_menhir_stack
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_menhir_stack[/* semv */1],
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _v_000 = _menhir_stack[/* semv */1];
      var _v = /* :: */[
        _v_000,
        /* [] */0
      ];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _v = List.rev(_menhir_stack[/* semv */1]);
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var _v_000 = _menhir_stack[/* semv */1];
      var _v_001 = match[/* semv */1];
      var _v = /* :: */[
        _v_000,
        _v_001
      ];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var _v = List.rev_append(_menhir_stack[/* semv */1], match[/* semv */1]);
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _v_000 = /* Positional */Block.__(0, [_menhir_stack[/* semv */1]]);
      var _v = /* :: */[
        _v_000,
        /* [] */0
      ];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var _v_000 = /* Positional */Block.__(0, [_menhir_stack[/* semv */1]]);
      var _v_001 = match[/* semv */1];
      var _v = /* :: */[
        _v_000,
        _v_001
      ];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var xs = match[/* semv */1];
      var pos = /* Positional */Block.__(0, [_menhir_stack[/* semv */1]]);
      var _v;
      if (xs) {
        var arg = xs[0];
        if (arg.tag) {
          var flag = arg[0];
          var match$1 = flag[/* payload */1];
          _v = typeof match$1 === "number" ? (
              match$1 !== 0 ? /* :: */[
                  pos,
                  /* :: */[
                    /* Flag */Block.__(1, [/* record */[
                          /* name */flag[/* name */0],
                          /* payload : Unresolved */0
                        ]]),
                    xs[1]
                  ]
                ] : Pervasives.failwith("unreachable: rev_arguments_and_positional, unresolved")
            ) : /* :: */[
              pos,
              xs
            ];
        } else {
          _v = Pervasives.failwith("unreachable: rev_arguments_and_positional, positional");
        }
      } else {
        _v = Pervasives.failwith("unreachable: rev_arguments_and_positional, empty xs");
      }
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_menhir_stack[/* semv */1],
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_menhir_stack[/* semv */1],
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv : [] */0,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var _v_000 = _menhir_stack[/* semv */1];
      var _v_001 = match[/* semv */1];
      var _v = /* :: */[
        _v_000,
        _v_001
      ];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var _v = Pervasives.$at(_menhir_stack[/* semv */1], match[/* semv */1]);
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var _v_000 = _menhir_stack[/* semv */1];
      var _v = /* :: */[
        _v_000,
        /* [] */0
      ];
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var _v_000 = _menhir_stack[/* semv */1];
      var _v_001 = match[/* semv */1];
      var _v = /* :: */[
        _v_000,
        _v_001
      ];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var _v = Pervasives.$at(_menhir_stack[/* semv */1], match[/* semv */1]);
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var _v_000 = _menhir_stack[/* semv */1];
      var _v_001 = match[/* semv */1];
      var _v = /* :: */[
        _v_000,
        _v_001
      ];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var _v = /* record */[/* expressions */$$Array.of_list(match[/* semv */1])];
      return /* record */[
              /* state */match[/* state */0],
              /* semv */_v,
              /* startp */match[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var explode = function (str) {
        var last = str.length - 1 | 0;
        if (str === "") {
          return /* [] */0;
        } else {
          return /* :: */[
                  $$String.make(1, Caml_string.get(str, 0)),
                  explode($$String.sub(str, 1, last))
                ];
        }
      };
      var xs = explode(_menhir_stack[/* semv */1]);
      var _v = List.map((function (x) {
              return /* Flag */Block.__(1, [/* record */[
                          /* name */x,
                          /* payload : Absent */1
                        ]]);
            }), xs);
      return /* record */[
              /* state */_menhir_stack[/* state */0],
              /* semv */_v,
              /* startp */_menhir_stack[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */_menhir_stack[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var match$1 = match[/* next */4];
      return /* record */[
              /* state */match$1[/* state */0],
              /* semv */match[/* semv */1],
              /* startp */match$1[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match$1[/* next */4]
            ];
    }),
  (function (_menhir_env) {
      var _menhir_stack = _menhir_env[/* stack */2];
      var match = _menhir_stack[/* next */4];
      var match$1 = match[/* next */4];
      var match$2 = match$1[/* next */4];
      var _v = AST.make_expression(match$1[/* semv */1], match[/* semv */1], _menhir_stack[/* semv */1]);
      return /* record */[
              /* state */match$2[/* state */0],
              /* semv */_v,
              /* startp */match$2[/* startp */2],
              /* endp */_menhir_stack[/* endp */3],
              /* next */match$2[/* next */4]
            ];
    })
];

var ET = MenhirLib.TableInterpreter.MakeEngineTable({
      token2terminal: token2terminal,
      error_terminal: 0,
      token2value: token2value,
      default_reduction: default_reduction,
      error: error,
      action: action,
      lhs: lhs,
      $$goto: $$goto,
      start: 2,
      semantic_action: semantic_action,
      $$Error: $$Error,
      trace: undefined
    });

var TI = MenhirLib.Engine.Make(ET);

var entry = TI.entry;

var start = TI.start;

var Symbols = { };

MenhirLib.InspectionTableInterpreter.Symbols(Symbols);

function terminal(t) {
  switch (t) {
    case 0 :
        return /* X */[/* T */Block.__(0, [/* T_error */0])];
    case 1 :
        return /* X */[/* T */Block.__(0, [/* T_URL_START */1])];
    case 2 :
        return /* X */[/* T */Block.__(0, [/* T_URL_REST */2])];
    case 3 :
        return /* X */[/* T */Block.__(0, [/* T_SEMICOLON */3])];
    case 4 :
        return /* X */[/* T */Block.__(0, [/* T_QUOTE_OPEN */4])];
    case 5 :
        return /* X */[/* T */Block.__(0, [/* T_QUOTE_ESCAPE */5])];
    case 6 :
        return /* X */[/* T */Block.__(0, [/* T_QUOTE_CLOSE */6])];
    case 7 :
        return /* X */[/* T */Block.__(0, [/* T_QUOTE_CHUNK */7])];
    case 8 :
        return /* X */[/* T */Block.__(0, [/* T_PIPE */8])];
    case 9 :
        return /* X */[/* T */Block.__(0, [/* T_PAREN_OPEN */9])];
    case 10 :
        return /* X */[/* T */Block.__(0, [/* T_PAREN_CLOSE */10])];
    case 11 :
        return /* X */[/* T */Block.__(0, [/* T_IDENTIFIER */11])];
    case 12 :
        return /* X */[/* T */Block.__(0, [/* T_FLAG_LONG_START */12])];
    case 13 :
        return /* X */[/* T */Block.__(0, [/* T_FLAGS_SHORT_START */13])];
    case 14 :
        return /* X */[/* T */Block.__(0, [/* T_ERR_UNEXPECTED_WHITESPACE */14])];
    case 15 :
        return /* X */[/* T */Block.__(0, [/* T_ERR_UNEXPECTED_QUOTE_ESCAPE */15])];
    case 16 :
        return /* X */[/* T */Block.__(0, [/* T_ERR_UNEXPECTED_QUOTE_CLOSE */16])];
    case 17 :
        return /* X */[/* T */Block.__(0, [/* T_ERR_UNEXPECTED_COMMENT_CLOSE */17])];
    case 18 :
        return /* X */[/* T */Block.__(0, [/* T_ERR_UNEXPECTED_CHARACTER */18])];
    case 19 :
        return /* X */[/* T */Block.__(0, [/* T_ERR_MISSING_DELIM_CLOSE */19])];
    case 20 :
        return /* X */[/* T */Block.__(0, [/* T_ERR_MISSING_COMMENT_CLOSE */20])];
    case 21 :
        return /* X */[/* T */Block.__(0, [/* T_EQUALS */21])];
    case 22 :
        return /* X */[/* T */Block.__(0, [/* T_EOF */22])];
    case 23 :
        return /* X */[/* T */Block.__(0, [/* T_COUNT */23])];
    case 24 :
        return /* X */[/* T */Block.__(0, [/* T_COMMENT_OPEN */24])];
    case 25 :
        return /* X */[/* T */Block.__(0, [/* T_COMMENT_CLOSE */25])];
    case 26 :
        return /* X */[/* T */Block.__(0, [/* T_COMMENT */26])];
    case 27 :
        return /* X */[/* T */Block.__(0, [/* T_COLON */27])];
    case 28 :
        return /* X */[/* T */Block.__(0, [/* T_BARE_DOUBLE_DASH */28])];
    default:
      throw [
            Caml_builtin_exceptions.assert_failure,
            /* tuple */[
              "parserAutomaton.ml",
              2416,
              12
            ]
          ];
  }
}

function nonterminal(nt) {
  switch (nt) {
    case 2 :
        return /* X */[/* N */Block.__(1, [/* N_unterminated_expression */0])];
    case 3 :
        return /* X */[/* N */Block.__(1, [/* N_subexpression */1])];
    case 4 :
        return /* X */[/* N */Block.__(1, [/* N_short_flags */2])];
    case 5 :
        return /* X */[/* N */Block.__(1, [/* N_script */3])];
    case 6 :
        return /* X */[/* N */Block.__(1, [/* N_rev_subquotation */4])];
    case 7 :
        return /* X */[/* N */Block.__(1, [/* N_rev_nonempty_subquotation */5])];
    case 8 :
        return /* X */[/* N */Block.__(1, [/* N_rev_nonempty_quotation */6])];
    case 9 :
        return /* X */[/* N */Block.__(1, [/* N_rev_arguments_nonempty */7])];
    case 10 :
        return /* X */[/* N */Block.__(1, [/* N_rev_arguments_and_positional */8])];
    case 11 :
        return /* X */[/* N */Block.__(1, [/* N_rev_arguments_and_flag */9])];
    case 12 :
        return /* X */[/* N */Block.__(1, [/* N_rev_arguments */10])];
    case 13 :
        return /* X */[/* N */Block.__(1, [/* N_quotation_chunk */11])];
    case 14 :
        return /* X */[/* N */Block.__(1, [/* N_quotation */12])];
    case 15 :
        return /* X */[/* N */Block.__(1, [/* N_optterm_nonempty_list_break_expression_chain_ */13])];
    case 16 :
        return /* X */[/* N */Block.__(1, [/* N_optterm_list_break_expression_chain_ */14])];
    case 17 :
        return /* X */[/* N */Block.__(1, [/* N_option_break_ */15])];
    case 18 :
        return /* X */[/* N */Block.__(1, [/* N_option_COUNT_ */16])];
    case 19 :
        return /* X */[/* N */Block.__(1, [/* N_noncommand_word */17])];
    case 20 :
        return /* X */[/* N */Block.__(1, [/* N_long_flag */18])];
    case 21 :
        return /* X */[/* N */Block.__(1, [/* N_list_COLON_ */19])];
    case 22 :
        return /* X */[/* N */Block.__(1, [/* N_expression_chain */20])];
    case 23 :
        return /* X */[/* N */Block.__(1, [/* N_expression */21])];
    case 24 :
        return /* X */[/* N */Block.__(1, [/* N_command */22])];
    case 25 :
        return /* X */[/* N */Block.__(1, [/* N_break */23])];
    case 26 :
        return /* X */[/* N */Block.__(1, [/* N__flags_short */24])];
    case 27 :
        return /* X */[/* N */Block.__(1, [/* N__flag_long */25])];
    default:
      throw [
            Caml_builtin_exceptions.assert_failure,
            /* tuple */[
              "parserAutomaton.ml",
              2474,
              12
            ]
          ];
  }
}

var include = MenhirLib.InspectionTableInterpreter.Make({
            token2terminal: token2terminal,
            error_terminal: 0,
            token2value: token2value,
            default_reduction: default_reduction,
            error: error,
            action: action,
            lhs: lhs,
            $$goto: $$goto,
            start: 2,
            semantic_action: semantic_action,
            $$Error: $$Error,
            trace: undefined
          })({
          terminal: terminal,
          nonterminal: nonterminal,
          rhs: /* tuple */[
            /* tuple */[
              8,
              "/\x0b\x1a\x18\x1a\x1d\x1c\x18\x1c\x1d\b\x18\x1d\x07-#.\x05-\x12\x058+77,'\x18\x04\x06:\x1d\x0703#\x1f-#-3\x1f\x11\x0e\x10\f\x13)\t\x13)\x13\t'\x15'\x17'\x17\x15\n\x11\x1b\x11\r\n\x0f\x1b\x0f\r\x0f\x0e!.5\x14-\x16+%1\x19"
            ],
            /* tuple */[
              8,
              "\0\x01\x02\x04\x06\b\n\x0b\f\r\x0e\x11\x12\x15\x15\x17\x18\x1b\x1c\x1e\x1f !!\"\"#$%'*,-../013568:;<=?ABDFHJKNR"
            ]
          ],
          lr0_core: /* tuple */[
            8,
            "\0\x01\x02\x03\x04\x05\x06\x07\b\t\n\x0b\f\r\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEF"
          ],
          lr0_items: /* tuple */[
            /* tuple */[
              16,
              "\0\0\x07\x01\x07\x02\x05\x81\x1b\x01\x0b\x81\x1b\x02\x16\x01\x1a\x81\x1a\x82\x06\x01\x06\x02\x06\x03\x1a\x83\x03\x81\x04\x81\x17\x01\x16\x81\x0f\x01\x17\x81\x10\x01\x0f\x02\x0f\x81\x17\x02\x19\x01\x18\x81\x18\x01\x19\x02\x18\x82\x18\x02\x16\x82\x04\x01\x1b\x03\t\x01\t\x02\b\x81\x01\x81\x01\x01\x01\x02\x01\x82\x02\x81\x02\x01\x02\x02\x02\x82\t\x81\n\x81\x12\x01\x13\x01\x12\x81\x11\x01\x13\x02\x12\x82\x1a\x01\b\x01\x07\x81\b\x02\n\x01\b\x03\x15\x81\x14\x01\x14\x02\x15\x01\x14\x81\x14\x82\x1b\x04\x13\x81\x11\x81\x06\x01\x05\x01\x03\x01\x05\x02\x05\x03\f\x81\0\x01\0\x80\0\x81\r\x81\x19\x81\x19\x82\r\x01\x0e\x81\x0e\x01\x06\x01\x0e\x02\x0e\x82\f\x81\x0e\x83"
            ],
            /* tuple */[
              8,
              "\0\x01\x02\x03\x04\x05\x06\x07\b\t\x0b\f\r\x0e\x0f\x10\x13\x14\x15\x16\x17\x18\x1b\x1c\x1d\x1e\x1f !\"#$&'(*+,-./2345789:<=?@ABCEFGHIJKLMNOPSTVW"
            ]
          ],
          lr0_incoming: /* tuple */[
            8,
            "\x008+\x05+0%\n\x14-\x12\x05\x16\x18\x07\x11\n\f\x0e\x10\r\x0f\x0e\r\x1b\x1b\x1d1\x04\x06\x18\x1a\x18\x1d\x1c\x18\x1d:\x07\t\x13\t)57,\x1d'\x15'\x17'\x19')-\b#.3/\0\x0b\x1f!.#-#3\x1f"
          ],
          nullable: "\0\b\xe4\0",
          first: /* tuple */[
            29,
            "\x18P\x03\x10B\x80\b\x82\x14\0D\0\x80\0\0\0@\0\f(\x01\x88 \0\0\x01\0\0\0\b\0\0\x02B\xe0\0R\x17\0\x02\x90\xb8\0\x14\x85\xc0\0\x82\x80\0\0 \0\0\x01\n\0\"\x18P\x01\x10\x80\0\0\0\0\0@\x90\xa0\0\x10\0\x80\0\0\0\0\b!@\x04A\n\0\"\bP\0\0\x80\0\0\0\x01\0\0\0\x10\0\0"
          ]
        })(ET)({ });

function script(lexer, lexbuf) {
  return Curry._3(entry, 61, lexer, lexbuf);
}

function expression(lexer, lexbuf) {
  return Curry._3(entry, 0, lexer, lexbuf);
}

function script$1(initial_position) {
  return Curry._2(start, 61, initial_position);
}

function expression$1(initial_position) {
  return Curry._2(start, 0, initial_position);
}

var Incremental = {
  script: script$1,
  expression: expression$1
};

var MenhirInterpreter_offer = TI.offer;

var MenhirInterpreter_resume = TI.resume;

var MenhirInterpreter_lexer_lexbuf_to_supplier = TI.lexer_lexbuf_to_supplier;

var MenhirInterpreter_loop = TI.loop;

var MenhirInterpreter_loop_handle = TI.loop_handle;

var MenhirInterpreter_loop_handle_undo = TI.loop_handle_undo;

var MenhirInterpreter_shifts = TI.shifts;

var MenhirInterpreter_acceptable = TI.acceptable;

var MenhirInterpreter_number = TI.number;

var MenhirInterpreter_production_index = TI.production_index;

var MenhirInterpreter_find_production = TI.find_production;

var MenhirInterpreter_stack = TI.stack;

var MenhirInterpreter_top = TI.top;

var MenhirInterpreter_pop_many = TI.pop_many;

var MenhirInterpreter_get = TI.get;

var MenhirInterpreter_current_state_number = TI.current_state_number;

var MenhirInterpreter_equal = TI.equal;

var MenhirInterpreter_positions = TI.positions;

var MenhirInterpreter_env_has_default_reduction = TI.env_has_default_reduction;

var MenhirInterpreter_state_has_default_reduction = TI.state_has_default_reduction;

var MenhirInterpreter_pop = TI.pop;

var MenhirInterpreter_force_reduction = TI.force_reduction;

var MenhirInterpreter_input_needed = TI.input_needed;

var MenhirInterpreter_compare_terminals = include.compare_terminals;

var MenhirInterpreter_compare_nonterminals = include.compare_nonterminals;

var MenhirInterpreter_compare_symbols = include.compare_symbols;

var MenhirInterpreter_compare_productions = include.compare_productions;

var MenhirInterpreter_compare_items = include.compare_items;

var MenhirInterpreter_incoming_symbol = include.incoming_symbol;

var MenhirInterpreter_items = include.items;

var MenhirInterpreter_lhs = include.lhs;

var MenhirInterpreter_rhs = include.rhs;

var MenhirInterpreter_nullable = include.nullable;

var MenhirInterpreter_first = include.first;

var MenhirInterpreter_xfirst = include.xfirst;

var MenhirInterpreter_foreach_terminal = include.foreach_terminal;

var MenhirInterpreter_foreach_terminal_but_error = include.foreach_terminal_but_error;

var MenhirInterpreter_feed = include.feed;

var MenhirInterpreter = {
  offer: MenhirInterpreter_offer,
  resume: MenhirInterpreter_resume,
  lexer_lexbuf_to_supplier: MenhirInterpreter_lexer_lexbuf_to_supplier,
  loop: MenhirInterpreter_loop,
  loop_handle: MenhirInterpreter_loop_handle,
  loop_handle_undo: MenhirInterpreter_loop_handle_undo,
  shifts: MenhirInterpreter_shifts,
  acceptable: MenhirInterpreter_acceptable,
  number: MenhirInterpreter_number,
  production_index: MenhirInterpreter_production_index,
  find_production: MenhirInterpreter_find_production,
  stack: MenhirInterpreter_stack,
  top: MenhirInterpreter_top,
  pop_many: MenhirInterpreter_pop_many,
  get: MenhirInterpreter_get,
  current_state_number: MenhirInterpreter_current_state_number,
  equal: MenhirInterpreter_equal,
  positions: MenhirInterpreter_positions,
  env_has_default_reduction: MenhirInterpreter_env_has_default_reduction,
  state_has_default_reduction: MenhirInterpreter_state_has_default_reduction,
  pop: MenhirInterpreter_pop,
  force_reduction: MenhirInterpreter_force_reduction,
  input_needed: MenhirInterpreter_input_needed,
  compare_terminals: MenhirInterpreter_compare_terminals,
  compare_nonterminals: MenhirInterpreter_compare_nonterminals,
  compare_symbols: MenhirInterpreter_compare_symbols,
  compare_productions: MenhirInterpreter_compare_productions,
  compare_items: MenhirInterpreter_compare_items,
  incoming_symbol: MenhirInterpreter_incoming_symbol,
  items: MenhirInterpreter_items,
  lhs: MenhirInterpreter_lhs,
  rhs: MenhirInterpreter_rhs,
  nullable: MenhirInterpreter_nullable,
  first: MenhirInterpreter_first,
  xfirst: MenhirInterpreter_xfirst,
  foreach_terminal: MenhirInterpreter_foreach_terminal,
  foreach_terminal_but_error: MenhirInterpreter_foreach_terminal_but_error,
  feed: MenhirInterpreter_feed
};

exports.$$Error = $$Error;
exports.script = script;
exports.expression = expression;
exports.MenhirInterpreter = MenhirInterpreter;
exports.Incremental = Incremental;
/* ET Not a pure module */
