'use strict';

var $$Array = require("bs-platform/lib/js/array.js");
var Curry = require("bs-platform/lib/js/curry.js");
var Lexer = require("./lexer.bs.js");
var Lexing = require("bs-platform/lib/js/lexing.js");
var $$String = require("bs-platform/lib/js/string.js");
var Sedlexing = require("bs-sedlex/src/lib/sedlexing.bs.js");
var Caml_array = require("bs-platform/lib/js/caml_array.js");
var Pervasives = require("bs-platform/lib/js/pervasives.js");
var Caml_exceptions = require("bs-platform/lib/js/caml_exceptions.js");
var ParserAutomaton = require("./parserAutomaton.bs.js");
var Caml_builtin_exceptions = require("bs-platform/lib/js/caml_builtin_exceptions.js");

function script(buf) {
  var match = Sedlexing.lexing_positions(Lexer.sedlex_of_buffer(buf));
  return /* record */[
          /* status */ParserAutomaton.Incremental.script(match[1]),
          /* buf */buf
        ];
}

function script_of_string(s) {
  var buf = Lexer.buffer_of_string(s);
  var match = Sedlexing.lexing_positions(Lexer.sedlex_of_buffer(buf));
  return /* record */[
          /* status */ParserAutomaton.Incremental.script(match[1]),
          /* buf */buf
        ];
}

function expression(buf) {
  var match = Sedlexing.lexing_positions(Lexer.sedlex_of_buffer(buf));
  return /* record */[
          /* status */ParserAutomaton.Incremental.expression(match[1]),
          /* buf */buf
        ];
}

function expression_of_string(s) {
  var buf = Lexer.buffer_of_string(s);
  var match = Sedlexing.lexing_positions(Lexer.sedlex_of_buffer(buf));
  return /* record */[
          /* status */ParserAutomaton.Incremental.expression(match[1]),
          /* buf */buf
        ];
}

var Break = Caml_exceptions.create("Incremental.Break");

function acceptable_token(cp) {
  var cp$1 = cp[/* status */0];
  var len = Lexer.example_tokens.length;
  var accepted_token = undefined;
  try {
    for(var i = 0; i <= len; ++i){
      var tok = Caml_array.caml_array_get(Lexer.example_tokens, i);
      if (Curry._3(ParserAutomaton.MenhirInterpreter.acceptable, cp$1, tok, Lexing.dummy_pos)) {
        accepted_token = tok;
      }
      throw Break;
    }
  }
  catch (exn){
    if (exn !== Break) {
      throw exn;
    }
    
  }
  var match = accepted_token;
  if (match !== undefined) {
    return match;
  } else {
    throw Caml_builtin_exceptions.not_found;
  }
}

function acceptable_tokens(cp) {
  var cp$1 = cp[/* status */0];
  var accepted_tokens = /* record */[/* contents : [] */0];
  $$Array.iter((function (tok) {
          if (Curry._3(ParserAutomaton.MenhirInterpreter.acceptable, cp$1, tok, Lexing.dummy_pos)) {
            accepted_tokens[0] = /* :: */[
              tok,
              accepted_tokens[0]
            ];
            return /* () */0;
          } else {
            return 0;
          }
        }), Lexer.example_tokens);
  return $$Array.of_list(accepted_tokens[0]);
}

function $$continue(accept, fail, cp) {
  var buf = cp[/* buf */1];
  var last_token = /* record */[/* contents : tuple */[
      /* EOF */7,
      Lexing.dummy_pos,
      Lexing.dummy_pos
    ]];
  var supplier = function (param) {
    last_token[0] = Lexer.next_loc(buf);
    return last_token[0];
  };
  var fail$1 = function (cp1, cp2) {
    return Curry._2(fail, /* record */[
                /* status */cp1,
                /* buf */buf
              ], /* record */[
                /* status */cp2,
                /* buf */buf
              ]);
  };
  return Curry._4(ParserAutomaton.MenhirInterpreter.loop_handle_undo, accept, fail$1, supplier, cp[/* status */0]);
}

function current_command(cp) {
  var menhir_cp = cp[/* status */0];
  if (typeof menhir_cp === "number" || menhir_cp.tag) {
    return ;
  } else {
    var env = menhir_cp[0];
    var _i = 0;
    while(true) {
      var i = _i;
      var match = Curry._2(ParserAutomaton.MenhirInterpreter.get, i, env);
      if (match !== undefined) {
        var match$1 = match;
        var match$2 = Curry._1(ParserAutomaton.MenhirInterpreter.incoming_symbol, match$1[0]);
        if (match$2.tag) {
          if (match$2[0] !== 22) {
            _i = i + 1 | 0;
            continue ;
          } else {
            return match$1[1];
          }
        } else {
          _i = i + 1 | 0;
          continue ;
        }
      } else {
        return ;
      }
    };
  }
}

function automaton_status_str(cp) {
  var tmp = cp[/* status */0];
  if (typeof tmp === "number") {
    return "Rejected";
  } else {
    switch (tmp.tag | 0) {
      case /* InputNeeded */0 :
          return "InputNeeded";
      case /* Shifting */1 :
          return "Shifting";
      case /* AboutToReduce */2 :
          return "AboutToReduce";
      case /* HandlingError */3 :
          return "HandlingError";
      case /* Accepted */4 :
          return "Accepted";
      
    }
  }
}

function element_incoming_symbol_category_str(param) {
  var match = Curry._1(ParserAutomaton.MenhirInterpreter.incoming_symbol, param[0]);
  if (match.tag) {
    return "Nonterminal";
  } else {
    return "Terminal";
  }
}

function incoming_symbol_category_str(cp) {
  var cp$1 = cp[/* status */0];
  var the_env;
  the_env = typeof cp$1 === "number" ? Pervasives.failwith("incoming_symbol_category: I don't know how to handle Rejected checkpoints") : (
      cp$1.tag === /* Accepted */4 ? Pervasives.failwith("incoming_symbol_category: I don't know how to handle Accepted checkpoints") : cp$1[0]
    );
  var match = Curry._1(ParserAutomaton.MenhirInterpreter.top, the_env);
  if (match !== undefined) {
    return element_incoming_symbol_category_str(match);
  }
  
}

function element_incoming_symbol_desc(param) {
  var match = Curry._1(ParserAutomaton.MenhirInterpreter.incoming_symbol, param[0]);
  if (match.tag) {
    switch (match[0]) {
      case /* N_unterminated_expression */0 :
          return /* tuple */[
                  "unterminated_expression",
                  "AST.expression"
                ];
      case /* N_subexpression */1 :
          return /* tuple */[
                  "subexpression",
                  "AST.expression"
                ];
      case /* N_short_flags */2 :
          return /* tuple */[
                  "short_flags",
                  "AST.arg list"
                ];
      case /* N_script */3 :
          return /* tuple */[
                  "script",
                  "AST.t"
                ];
      case /* N_rev_subquotation */4 :
          return /* tuple */[
                  "rev_subquotation",
                  "string list"
                ];
      case /* N_rev_nonempty_subquotation */5 :
          return /* tuple */[
                  "rev_nonempty_subquotation",
                  "string list"
                ];
      case /* N_rev_nonempty_quotation */6 :
          return /* tuple */[
                  "rev_nonempty_quotation",
                  "string list"
                ];
      case /* N_rev_arguments_nonempty */7 :
          return /* tuple */[
                  "rev_arguments_nonempty",
                  "AST.arg list"
                ];
      case /* N_rev_arguments_and_positional */8 :
          return /* tuple */[
                  "rev_arguments_and_positional",
                  "AST.arg list"
                ];
      case /* N_rev_arguments_and_flag */9 :
          return /* tuple */[
                  "rev_arguments_and_flag",
                  "AST.arg list"
                ];
      case /* N_rev_arguments */10 :
          return /* tuple */[
                  "rev_arguments",
                  "AST.arg list"
                ];
      case /* N_quotation_chunk */11 :
          return /* tuple */[
                  "quotation_chunk",
                  "string"
                ];
      case /* N_quotation */12 :
          return /* tuple */[
                  "quotation",
                  "string"
                ];
      case /* N_optterm_nonempty_list_break_expression_chain_ */13 :
          return /* tuple */[
                  "optterm_nonempty_list_break_expression_chain_",
                  "AST.expression list"
                ];
      case /* N_optterm_list_break_expression_chain_ */14 :
          return /* tuple */[
                  "optterm_list_break_expression_chain_",
                  "AST.expression list"
                ];
      case /* N_option_break_ */15 :
          return /* tuple */[
                  "option_break_",
                  "unit option"
                ];
      case /* N_option_COUNT_ */16 :
          return /* tuple */[
                  "option_COUNT_",
                  "string option"
                ];
      case /* N_noncommand_word */17 :
          return /* tuple */[
                  "noncommand_word",
                  "string AST.or_subexpr"
                ];
      case /* N_long_flag */18 :
          return /* tuple */[
                  "long_flag",
                  "AST.arg"
                ];
      case /* N_list_COLON_ */19 :
          return /* tuple */[
                  "list_COLON_",
                  "unit list"
                ];
      case /* N_expression_chain */20 :
          return /* tuple */[
                  "expression_chain",
                  "AST.expression"
                ];
      case /* N_expression */21 :
          return /* tuple */[
                  "expression",
                  "AST.expression"
                ];
      case /* N_command */22 :
          return /* tuple */[
                  "command",
                  "string AST.or_subexpr"
                ];
      case /* N_break */23 :
          return /* tuple */[
                  "break",
                  "unit"
                ];
      case /* N__flags_short */24 :
          return /* tuple */[
                  "_flags_short",
                  "string"
                ];
      case /* N__flag_long */25 :
          return /* tuple */[
                  "_flag_long",
                  "string"
                ];
      
    }
  } else {
    switch (match[0]) {
      case /* T_error */0 :
          return /* tuple */[
                  "error",
                  "unit"
                ];
      case /* T_URL_START */1 :
          return /* tuple */[
                  "URL_START",
                  "string"
                ];
      case /* T_URL_REST */2 :
          return /* tuple */[
                  "URL_REST",
                  "string"
                ];
      case /* T_SEMICOLON */3 :
          return /* tuple */[
                  "SEMICOLON",
                  "unit"
                ];
      case /* T_QUOTE_OPEN */4 :
          return /* tuple */[
                  "QUOTE_OPEN",
                  "string"
                ];
      case /* T_QUOTE_ESCAPE */5 :
          return /* tuple */[
                  "QUOTE_ESCAPE",
                  "string"
                ];
      case /* T_QUOTE_CLOSE */6 :
          return /* tuple */[
                  "QUOTE_CLOSE",
                  "string"
                ];
      case /* T_QUOTE_CHUNK */7 :
          return /* tuple */[
                  "QUOTE_CHUNK",
                  "string"
                ];
      case /* T_PIPE */8 :
          return /* tuple */[
                  "PIPE",
                  "unit"
                ];
      case /* T_PAREN_OPEN */9 :
          return /* tuple */[
                  "PAREN_OPEN",
                  "unit"
                ];
      case /* T_PAREN_CLOSE */10 :
          return /* tuple */[
                  "PAREN_CLOSE",
                  "unit"
                ];
      case /* T_IDENTIFIER */11 :
          return /* tuple */[
                  "IDENTIFIER",
                  "string"
                ];
      case /* T_FLAG_LONG_START */12 :
          return /* tuple */[
                  "FLAG_LONG_START",
                  "string"
                ];
      case /* T_FLAGS_SHORT_START */13 :
          return /* tuple */[
                  "FLAGS_SHORT_START",
                  "string"
                ];
      case /* T_ERR_UNEXPECTED_WHITESPACE */14 :
          return /* tuple */[
                  "ERR_UNEXPECTED_WHITESPACE",
                  "string"
                ];
      case /* T_ERR_UNEXPECTED_QUOTE_ESCAPE */15 :
          return /* tuple */[
                  "ERR_UNEXPECTED_QUOTE_ESCAPE",
                  "string * string"
                ];
      case /* T_ERR_UNEXPECTED_QUOTE_CLOSE */16 :
          return /* tuple */[
                  "ERR_UNEXPECTED_QUOTE_CLOSE",
                  "string * string"
                ];
      case /* T_ERR_UNEXPECTED_COMMENT_CLOSE */17 :
          return /* tuple */[
                  "ERR_UNEXPECTED_COMMENT_CLOSE",
                  "string"
                ];
      case /* T_ERR_UNEXPECTED_CHARACTER */18 :
          return /* tuple */[
                  "ERR_UNEXPECTED_CHARACTER",
                  "int * string"
                ];
      case /* T_ERR_MISSING_DELIM_CLOSE */19 :
          return /* tuple */[
                  "ERR_MISSING_DELIM_CLOSE",
                  "string * string"
                ];
      case /* T_ERR_MISSING_COMMENT_CLOSE */20 :
          return /* tuple */[
                  "ERR_MISSING_COMMENT_CLOSE",
                  "string"
                ];
      case /* T_EQUALS */21 :
          return /* tuple */[
                  "EQUALS",
                  "unit"
                ];
      case /* T_EOF */22 :
          return /* tuple */[
                  "EOF",
                  "unit"
                ];
      case /* T_COUNT */23 :
          return /* tuple */[
                  "COUNT",
                  "string"
                ];
      case /* T_COMMENT_OPEN */24 :
          return /* tuple */[
                  "COMMENT_OPEN",
                  "unit"
                ];
      case /* T_COMMENT_CLOSE */25 :
          return /* tuple */[
                  "COMMENT_CLOSE",
                  "unit"
                ];
      case /* T_COMMENT */26 :
          return /* tuple */[
                  "COMMENT",
                  "string"
                ];
      case /* T_COLON */27 :
          return /* tuple */[
                  "COLON",
                  "unit"
                ];
      case /* T_BARE_DOUBLE_DASH */28 :
          return /* tuple */[
                  "BARE_DOUBLE_DASH",
                  "unit"
                ];
      
    }
  }
}

function element_incoming_symbol_type_str(el) {
  return element_incoming_symbol_desc(el)[1];
}

function incoming_symbol_type_str(cp) {
  var cp$1 = cp[/* status */0];
  var the_env;
  the_env = typeof cp$1 === "number" ? Pervasives.failwith("incoming_symbol_type: I don't know how to handle Rejected checkpoints") : (
      cp$1.tag === /* Accepted */4 ? Pervasives.failwith("incoming_symbol_type: I don't know how to handle Accepted checkpoints") : cp$1[0]
    );
  var match = Curry._1(ParserAutomaton.MenhirInterpreter.top, the_env);
  if (match !== undefined) {
    return element_incoming_symbol_type_str(match);
  }
  
}

function element_incoming_symbol_str(el) {
  return element_incoming_symbol_desc(el)[0];
}

function incoming_symbol_str(cp) {
  var cp$1 = cp[/* status */0];
  var the_env;
  the_env = typeof cp$1 === "number" ? Pervasives.failwith("incoming_symbol: I don't know how to handle Rejected checkpoints") : (
      cp$1.tag === /* Accepted */4 ? Pervasives.failwith("incoming_symbol: I don't know how to handle Accepted checkpoints") : cp$1[0]
    );
  var match = Curry._1(ParserAutomaton.MenhirInterpreter.top, the_env);
  if (match !== undefined) {
    return element_incoming_symbol_str(match);
  }
  
}

function element_incoming_symbol_desc_str(el) {
  var cat = element_incoming_symbol_category_str(el);
  var match = element_incoming_symbol_desc(el);
  return $$String.concat("", /* :: */[
              match[0],
              /* :: */[
                " : (",
                /* :: */[
                  match[1],
                  /* :: */[
                    ") ",
                    /* :: */[
                      cat,
                      /* [] */0
                    ]
                  ]
                ]
              ]
            ]);
}

function debug_checkpoint(cp) {
  var menhir_cp = cp[/* status */0];
  if (typeof menhir_cp === "number" || menhir_cp.tag) {
    return Pervasives.failwith("supposed to be InputNeeded");
  } else {
    var env = menhir_cp[0];
    var _i = 0;
    while(true) {
      var i = _i;
      var match = Curry._2(ParserAutomaton.MenhirInterpreter.get, i, env);
      if (match !== undefined) {
        console.log(element_incoming_symbol_desc_str(match));
        _i = i + 1 | 0;
        continue ;
      } else {
        return /* () */0;
      }
    };
  }
}

function get_before(cp, idx) {
  var cp$1 = cp[/* status */0];
  var the_env;
  the_env = typeof cp$1 === "number" ? Pervasives.failwith("get_before: I don't know how to handle Rejected checkpoints") : (
      cp$1.tag === /* Accepted */4 ? Pervasives.failwith("get_before: I don't know how to handle Accepted checkpoints") : cp$1[0]
    );
  return Curry._2(ParserAutomaton.MenhirInterpreter.get, idx, the_env);
}

function get_after(cp, idx) {
  var cp$1 = cp[/* status */0];
  var the_env;
  the_env = typeof cp$1 === "number" || cp$1.tag !== /* Shifting */1 ? Pervasives.failwith("get_after: This function is only relevant for Shifting checkpoints") : cp$1[1];
  return Curry._2(ParserAutomaton.MenhirInterpreter.get, idx, the_env);
}

exports.script = script;
exports.script_of_string = script_of_string;
exports.expression = expression;
exports.expression_of_string = expression_of_string;
exports.$$continue = $$continue;
exports.acceptable_token = acceptable_token;
exports.acceptable_tokens = acceptable_tokens;
exports.current_command = current_command;
exports.automaton_status_str = automaton_status_str;
exports.element_incoming_symbol_category_str = element_incoming_symbol_category_str;
exports.incoming_symbol_category_str = incoming_symbol_category_str;
exports.element_incoming_symbol_type_str = element_incoming_symbol_type_str;
exports.incoming_symbol_type_str = incoming_symbol_type_str;
exports.element_incoming_symbol_str = element_incoming_symbol_str;
exports.incoming_symbol_str = incoming_symbol_str;
exports.get_before = get_before;
exports.get_after = get_after;
exports.debug_checkpoint = debug_checkpoint;
/* Lexer Not a pure module */
