
(* The type of tokens. *)

type token = Tokens.token

(* This exception is raised by the monolithic API functions. *)

exception Error

(* The monolithic API. *)

val script: (Lexing.lexbuf -> token) -> Lexing.lexbuf -> (AST.t)

val expression: (Lexing.lexbuf -> token) -> Lexing.lexbuf -> (AST.expression)

module MenhirInterpreter : sig
  
  (* The incremental API. *)
  
  include MenhirLib.IncrementalEngine.INCREMENTAL_ENGINE
    with type token = token
  
  (* The indexed type of terminal symbols. *)
  
  type 'a terminal = 'a Tokens.terminal
  
  (* The indexed type of nonterminal symbols. *)
  
  type _ nonterminal = 
    | N_unterminated_expression : (AST.expression) nonterminal
    | N_subexpression : (AST.expression) nonterminal
    | N_short_flags : (AST.arg list) nonterminal
    | N_script : (AST.t) nonterminal
    | N_rev_subquotation : (string list) nonterminal
    | N_rev_nonempty_subquotation : (string list) nonterminal
    | N_rev_nonempty_quotation : (string list) nonterminal
    | N_rev_arguments_nonempty : (AST.arg list) nonterminal
    | N_rev_arguments_and_positional : (AST.arg list) nonterminal
    | N_rev_arguments_and_flag : (AST.arg list) nonterminal
    | N_rev_arguments : (AST.arg list) nonterminal
    | N_quotation_chunk : (string) nonterminal
    | N_quotation : (string) nonterminal
    | N_optterm_nonempty_list_break_expression_chain_ : (AST.expression list) nonterminal
    | N_optterm_list_break_expression_chain_ : (AST.expression list) nonterminal
    | N_option_break_ : (unit option) nonterminal
    | N_option_COUNT_ : (string option) nonterminal
    | N_noncommand_word : (string AST.or_subexpr) nonterminal
    | N_long_flag : (AST.arg) nonterminal
    | N_list_COLON_ : (unit list) nonterminal
    | N_expression_chain : (AST.expression) nonterminal
    | N_expression : (AST.expression) nonterminal
    | N_command : (string AST.or_subexpr) nonterminal
    | N_break : (unit) nonterminal
    | N__flags_short : (string) nonterminal
    | N__flag_long : (string) nonterminal
  
  (* The inspection API. *)
  
  include MenhirLib.IncrementalEngine.INSPECTION
    with type 'a lr1state := 'a lr1state
    with type production := production
    with type 'a terminal := 'a terminal
    with type 'a nonterminal := 'a nonterminal
    with type 'a env := 'a env
  
end

(* The entry point(s) to the incremental API. *)

module Incremental : sig
  
  val script: Lexing.position -> (AST.t) MenhirInterpreter.checkpoint
  
  val expression: Lexing.position -> (AST.expression) MenhirInterpreter.checkpoint
  
end
